@@ -5,10 +5,10 @@ use warnings;
 use 5.008001;
 use FFI::Platypus 0.56;
 use FFI::CheckLib ();
-use NewRelic::Agent::FFI::Procedural;
+use NewRelic::Agent::FFI::Procedural ();
 
 # ABSTRACT: Perl Agent for NewRelic APM
-our $VERSION = '0.03'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
 
 sub new
@@ -41,34 +41,25 @@ sub new
   }, $class;
 }
 
-my $ffi = $NewRelic::Agent::FFI::Procedural::ffi;
-my $newrelic_basic_literal_replacement_obfuscator = $ffi->find_symbol('newrelic_basic_literal_replacement_obfuscator');
-
 
 sub embed_collector
 {
-  my($self) = @_;
-  my $newrelic_message_handler = $ffi->find_symbol('newrelic_message_handler');
-  if($newrelic_message_handler)
-  {
-    $ffi->function('newrelic_register_message_handler' => ['opaque'] => 'void')->call($newrelic_message_handler);
-  }
-  else
-  {
-    Carp::croak("unable to find newrelic_message_handler");
-  }
+  NewRelic::Agent::FFI::Procedural::newrelic_register_message_handler(
+    NewRelic::Agent::FFI::Procedural::newrelic_message_handler
+  );
 }
 
 
-$ffi->attach( [ newrelic_init => 'init' ] => [ 'string', 'string', 'string', 'string' ] => 'int' => sub {
-  my($xsub, $self) = @_;
-  $xsub->(
+sub init
+{
+  my($self) = @_;
+  NewRelic::Agent::FFI::Procedural::newrelic_init(
     $self->get_license_key,
     $self->get_app_name,
     $self->get_app_language,
     $self->get_app_language_version,
   );
-});
+}
 
 
 sub begin_transaction
@@ -158,9 +149,12 @@ sub begin_generic_segment
 }
 
 
-$ffi->attach( [ newrelic_segment_datastore_begin => 'begin_datastore_segment' ] => [ 'long', 'long', 'string', 'string', 'string', 'string', 'opaque' ] => 'long' => sub {
-  $_[0]->(@_[2,3,4,5,6,7], $newrelic_basic_literal_replacement_obfuscator);
-});
+sub begin_datastore_segment
+{
+  NewRelic::Agent::FFI::Procedural::newrelic_segment_datastore_begin(
+    @_[1,2,3,4,5,6], NewRelic::Agent::FFI::Procedural::newrelic_basic_literal_replacement_obfuscator(),
+  )
+}
 
 
 sub begin_external_segment
@@ -182,6 +176,7 @@ sub get_app_name { shift->{app_name} }
 sub get_app_language { shift->{app_language} }
 sub get_app_language_version { shift->{app_language_version} }
 
+
 1;
 
 __END__
@@ -196,7 +191,7 @@ NewRelic::Agent::FFI - Perl Agent for NewRelic APM
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -443,11 +438,15 @@ Get the language name (usually C<perl>).
 
 Get the language version.
 
-=head2 CAVEATS
+=head1 SEE ALSO
 
-This module attempts to replicate the same interface as L<NewRelic::Agent>, and this module includes a superset of the same tests.  
-Unfortunately, the existing test suite for L<NewRelic::Agent> is completely insufficient to have a high degree of confidence that
-either module works.
+=over 4
+
+=item L<NewRelic::Agent::FFI::Procedural>
+
+Procedural interface, recommended over this one.
+
+=back
 
 =head1 AUTHOR
 
